@@ -431,6 +431,8 @@ class Move(ModelSQL, ModelView):
 
         if isinstance(uom, (int, long)):
             uom = uom_obj.browse(uom)
+        if isinstance(currency, (int, long)):
+            currency = currency_obj.browse(currency)
         if isinstance(company, (int, long)):
             company = company_obj.browse(company)
 
@@ -491,18 +493,20 @@ class Move(ModelSQL, ModelView):
         product = product_obj.browse(vals['product'])
         if vals.get('state') == 'done':
             vals['effective_date'] = effective_date
+            currency_id = vals.get('currency', self.default_currency())
+            company_id = vals.get('company', self.default_company())
             from_location = location_obj.browse(vals['from_location'])
             to_location = location_obj.browse(vals['to_location'])
             if from_location.type == 'supplier' \
                     and product.cost_price_method == 'average':
                 self._update_product_cost_price(vals['product'],
                         vals['quantity'], vals['uom'], vals['unit_price'],
-                        vals['currency'], vals['company'], effective_date)
+                        currency_id, company_id, effective_date)
             if to_location.type == 'supplier' \
                     and product.cost_price_method == 'average':
                 self._update_product_cost_price(vals['product'],
                         -vals['quantity'], vals['uom'], vals['unit_price'],
-                        vals['currency'], vals['company'], effective_date)
+                        currency_id, company_id, effective_date)
             if not vals.get('cost_price'):
                 # Re-read product to get the updated cost_price
                 product = product_obj.browse(vals['product'])
